// File-to-Resource 1.0 - CLI edition
// by Gabriele Biffi - www.biffuz.it
// Distributed under the terms of the MIT license.

// Standard C headers
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

// BeOS headers
#include <File.h>
#include <Resources.h>
#include <TypeConstants.h>

void PrintUsage(void)
{
	puts(
		"File-To-Resource 1.0 by Gabriele Biffi - www.biffuz.it\n\n"
		"Add the input file's content (or stdin stream) to a resource file.\n\n"
		"Usage:\n\n"
		"filetores output_file -i id [-n name] [input_file]\n\n"
		"\t-i id\tThe ID of the resource.\n"
		"\t-n name\tThe name of the resource.\n\n"
		"\tID can be decimal, octal (must begin with 0), or hexadecimal (must\n"
		"\tbegin with 0x). The type of the added resource will be B_RAW_TYPE.\n"
		"\tCurrent file size limit is 32 Mb.\n\n"
		"Examples:\n\n"
		"\tfiletores MyPictures.rsrc -i 0xFF -n MyDog 'My dog.jpg'\n\n"
		"\tAdd the file \"My dog.jpg\" to the resource file \"MyPictues.rsrc\"\n"
		"\twith ID 255 (0xFF in hex) and name \"MyDog\".\n\n"		
		"\tgzip -c MyBook.txt | filetores MyResource.rsrc -i 077\n\n"
		"\tCompress MyBook.txt and then add it to \"MyResources.rsrc\"\n"
		"\twith ID 63 (077 in octal).\n"
	);
}

int main(int argn, char *argv[])
{
	char *input_file_name, *output_file_name, *name, *id_check;
	bool have_id, have_input, have_name, have_output;
	BResources *res_file;
	BFile *output_file;
	FILE *input_file;
	size_t length;
	void *data;
	int32 id;
	int i;

	// If no arguments are given, print the help and exit
	if (argn < 2) {
		PrintUsage();
		return 0;
	}

	// Read the arguments
	i = 1;
	have_id = false;
	have_name = false;
	have_output = false;
	name = NULL;
	while (i < argn) {
		if (strcmp(argv[i], "-h") == 0 || strcmp(argv[i], "--help") == 0) {
			PrintUsage();
			return 0;
		}
		else if (strcmp(argv[i], "-i") == 0) {
			if (have_id || i == argn - 1) {
				fprintf(stderr, "filetores: error: bad arguments\n");
				return -1;
			}
			have_id = true;
			id = static_cast<int32>(strtol(argv[++i], &id_check, 0));
			if (*id_check != 0) {
				fprintf(stderr, "filetores: error: bad arguments\n");
				return -1;
			}
		}
		else if (strcmp(argv[i], "-n") == 0) {
			if (have_name || i == argn - 1) {
				fprintf(stderr, "filetores: error: bad arguments\n");
				return -1;
			}
			have_name = true;
			name = argv[++i];
		}
		else {
			if (have_output) {
				if (have_input) {
					fprintf(stderr, "filetores: error: bad arguments\n");
					return -1;
				}
				have_input = true;
				input_file_name = argv[i];
			}
			else {
				have_output = true;
				output_file_name = argv[i];
			}
		}
		i++;
	}

	// Check the arguments
	if (!have_id || !have_output) {
		fprintf(stderr, "filetores: error: bad arguments\n");
		return -1;
	}

	// Open input file
	if (have_input) {
		input_file = fopen(input_file_name, "rb");
		if (input_file == NULL) {
			fprintf(stderr, "filetores: error: could not open input file %s\n", input_file_name);
			return -2;
		}
	}
	else
		input_file = stdin;

	// Open resource file
	output_file = new BFile(output_file_name, B_READ_WRITE | B_CREATE_FILE);
	if (output_file->InitCheck() != B_OK) {
		fprintf(stderr, "filetores: error: could not open output file %s\n", output_file_name);
		if (have_input)
			fclose(input_file);
		return -3;
	}

	// Initialize resources
	res_file = new BResources();
	if (res_file->SetTo(output_file) != B_OK) {
		fprintf(stderr, "filetores: error: output file %s is not a resources file\n", output_file_name);
		if (have_input)
			fclose(input_file);
		return -4;
	}
	delete output_file;	

	// Check type and ID
	if (res_file->HasResource(B_RAW_TYPE, id)) {
		fprintf(stderr, "filetores: error: the resource file already has a resource of type B_RAW_TYPE and ID %d\n", static_cast<int>(id));
		if (have_input)
			fclose(input_file);
		return -5;
	}

	// Read file data
	data = malloc(32*1024*1024);
	length = fread(data, 1, 32*1024*1024, input_file);

	// Add resource
	if (res_file->AddResource(B_RAW_TYPE, id, data, length, name) != B_OK) {
		fprintf(stderr, "filetores: error: add resource failed\n");
		if (have_input)
			fclose(input_file);
		free(data);
		return -6;
	}
	res_file->Sync();

	// Close files and free memory
	delete res_file;
	if (have_input)
		fclose(input_file);
	free(data);

	return 0;
}
