// Resource-to-File 1.0 - CLI edition
// by Gabriele Biffi - www.biffuz.it
// Distributed under the terms of the MIT license.

// Standard C headers
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

// BeOS headers
#include <File.h>
#include <Resources.h>
#include <TypeConstants.h>

void PrintUsage(void)
{
	puts(
		"Resource-To-File 1.0 by Gabriele Biffi - www.biffuz.it\n\n"
		"Extract a resource from a resource file and writes it to a file or stdout.\n\n"
		"Usage:\n\n"
		"filetores input_file -i id [output_file]\n\n"
		"\t-i id\tThe ID of the resource.\n\n"
		"\tID can be decimal, octal (must begin with 0), or hexadecimal (must\n"
		"\tbegin with 0x). Currently extract only resources of type B_RAW_TYPE.\n\n"
		"Examples:\n\n"
		"\trestofile MyPictures.rsrc -i 0xFF 'My dog.jpg'\n\n"
		"\tExtract the resource 255 (0xFF in hex) from the resource file \"MyPictues.rsrc\"\n"
		"\tand saves it to the file \"My dog.jpg\".\n\n"		
		"\trestofile MyResource.rsrc -i 077 | gunzip -c > MyBook.txt\n\n"
		"\tExtract the resource 63 (077 in octal) from \"MyResources.rsrc\" and\n"
		"\tgunzip it.\n"
	);
}

int main(int argn, char *argv[])
{
	char *input_file_name, *output_file_name, *id_check;
	bool have_id, have_input, have_output;
	BResources *res_file;
	BFile *input_file;
	FILE *output_file;
	const void *data;
	size_t length;
	int32 id;
	int i;

	// If no arguments are given, print the help and exit
	if (argn < 2) {
		PrintUsage();
		return 0;
	}

	// Read the arguments
	i = 1;
	have_id = false;
	have_output = false;
	while (i < argn) {
		if (strcmp(argv[i], "-h") == 0 || strcmp(argv[i], "--help") == 0) {
			PrintUsage();
			return 0;
		}
		else if (strcmp(argv[i], "-i") == 0) {
			if (have_id || i == argn - 1) {
				fprintf(stderr, "restofile: error: bad arguments\n");
				return -1;
			}
			have_id = true;
			id = static_cast<int32>(strtol(argv[++i], &id_check, 0));
			if (*id_check != 0) {
				fprintf(stderr, "restofile: error: bad arguments\n");
				return -1;
			}
		}
		else {
			if (have_input) {
				if (have_output) {
					fprintf(stderr, "restofile: error: bad arguments\n");
					return -1;
				}
				have_output = true;
				output_file_name = argv[i];
			}
			else {
				have_input = true;
				input_file_name = argv[i];
			}
		}
		i++;
	}

	// Check the arguments
	if (!have_id || !have_input) {
		fprintf(stderr, "restofile: error: bad arguments\n");
		return -1;
	}

	// Open resource file
	input_file = new BFile(input_file_name, B_READ_ONLY);
	if (input_file->InitCheck() != B_OK) {
		fprintf(stderr, "restofile: error: could not open input file %s\n", input_file_name);
		return -2;
	}

	// Initialize resources
	res_file = new BResources();
	if (res_file->SetTo(input_file) != B_OK) {
		fprintf(stderr, "restofile: error: input file %s is not a resources file\n", input_file_name);
		delete input_file;
		return -3;
	}
	delete input_file;	

	// Check type and ID
	if (!(res_file->HasResource(B_RAW_TYPE, id))) {
		fprintf(stderr, "restofile: error: can't find a resource of type B_RAW_TYPE and ID %d\n", static_cast<int>(id));
		delete res_file;
		return -4;
	}

	// Load data
	data = res_file->LoadResource(B_RAW_TYPE, id, &length);
	if (data == NULL) {
		fprintf(stderr, "restofile: error: load resource failed\n");
		delete res_file;
		return -5;
	}

	// Open input file
	if (have_output) {
		output_file = fopen(output_file_name, "wb");
		if (output_file == NULL) {
			fprintf(stderr, "restofile: error: could not open output file %s\n", output_file_name);
			return -6;
		}
	}
	else
		output_file = stdout;

	// Write data
	if (fwrite(data, 1, length, output_file) != length) {
		fprintf(stderr, "restofile: error: write failed\n");
		if (have_output)
			fclose(output_file);
		delete res_file;
		return -7;
	}

	// Close files
	delete res_file;
	if (have_output)
		fclose(output_file);

	return 0;
}
